"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_1 = require("@teambit/graphql");
const harmony_1 = require("@teambit/harmony");
const react_router_1 = require("@teambit/react-router");
const react_1 = __importDefault(require("react"));
const react_dom_1 = __importDefault(require("react-dom"));
const compose_1 = require("./compose");
const ui_aspect_1 = require("./ui.aspect");
const client_context_1 = require("./ui/client-context");
// import * as serviceWorker from './serviceWorker';
// If you want your app to work offline and load faster, you can change
// unregister() to register() below. Note this comes with some pitfalls.
// Learn more about service workers: https://bit.ly/CRA-PWA
// serviceWorker.unregister();
/**
 * extension
 */
class UiUI {
    constructor(
    /**
     * GraphQL extension.
     */
    graphql, 
    /**
     * react-router extension.
     */
    router, 
    /**
     * ui root registry.
     */
    uiRootSlot, 
    /** slot for overlay ui elements */
    hudSlot, 
    /** slot for context provider elements */
    contextSlot) {
        this.graphql = graphql;
        this.router = router;
        this.uiRootSlot = uiRootSlot;
        this.hudSlot = hudSlot;
        this.contextSlot = contextSlot;
        /** adds elements to the Heads Up Display */
        this.registerHudItem = (element) => {
            this.hudSlot.register(element);
        };
    }
    render(rootExtension) {
        return __awaiter(this, void 0, void 0, function* () {
            const GraphqlProvider = this.graphql.getProvider;
            const rootFactory = this.getRoot(rootExtension);
            if (!rootFactory)
                throw new Error(`root: ${rootExtension} was not found`);
            const uiRoot = rootFactory();
            const routes = this.router.renderRoutes(uiRoot.routes);
            const hudItems = this.hudSlot.values();
            const contexts = this.contextSlot.values();
            react_dom_1.default.render(react_1.default.createElement(GraphqlProvider, null,
                react_1.default.createElement(client_context_1.ClientContext, null,
                    react_1.default.createElement(compose_1.Compose, { components: contexts },
                        hudItems,
                        routes))), document.getElementById('root'));
        });
    }
    // ** adds global context at the ui root */
    registerContext(context) {
        this.contextSlot.register(context);
    }
    registerRoot(uiRoot) {
        return this.uiRootSlot.register(uiRoot);
    }
    getRoot(rootExtension) {
        return this.uiRootSlot.get(rootExtension);
    }
    static provider([graphql, router], config, [uiRootSlot, hudSlot, contextSlot]) {
        return __awaiter(this, void 0, void 0, function* () {
            return new UiUI(graphql, router, uiRootSlot, hudSlot, contextSlot);
        });
    }
}
exports.UiUI = UiUI;
UiUI.slots = [harmony_1.Slot.withType(), harmony_1.Slot.withType(), harmony_1.Slot.withType()];
UiUI.dependencies = [graphql_1.GraphqlAspect, react_router_1.ReactRouterAspect];
UiUI.runtime = ui_aspect_1.UIRuntime;
ui_aspect_1.UIAspect.addRuntime(UiUI);
//# sourceMappingURL=ui.ui.runtime.js.map